'use strict';

Object.freeze({});
/**
 * Make a map and return a function for checking if a key
 * is in that map.
 */
function makeMap(str, expectsLowerCase) {
    var map = Object.create(null);
    var list = str.split(',');
    for (var i = 0; i < list.length; i++) {
        map[list[i]] = true;
    }
    return expectsLowerCase ? function (val) { return map[val.toLowerCase()]; } : function (val) { return map[val]; };
}
/**
 * Check if a tag is a built-in tag.
 */
makeMap('slot,component', true);
/**
 * Check if an attribute is a reserved attribute.
 */
makeMap('key,ref,slot,slot-scope,is');

makeMap('accept,accept-charset,accesskey,action,align,alt,async,autocomplete,' +
    'autofocus,autoplay,autosave,bgcolor,border,buffered,challenge,charset,' +
    'checked,cite,class,code,codebase,color,cols,colspan,content,' +
    'contenteditable,contextmenu,controls,coords,data,datetime,default,' +
    'defer,dir,dirname,disabled,download,draggable,dropzone,enctype,for,' +
    'form,formaction,headers,height,hidden,high,href,hreflang,http-equiv,' +
    'icon,id,ismap,itemprop,keytype,kind,label,lang,language,list,loop,low,' +
    'manifest,max,maxlength,media,method,GET,POST,min,multiple,email,file,' +
    'muted,name,novalidate,open,optimum,pattern,ping,placeholder,poster,' +
    'preload,radiogroup,readonly,rel,required,reversed,rows,rowspan,sandbox,' +
    'scope,scoped,seamless,selected,shape,size,type,text,password,sizes,span,' +
    'spellcheck,src,srcdoc,srclang,srcset,start,step,style,summary,tabindex,' +
    'target,title,usemap,value,width,wrap');
var isJS = function (file) { return /\.js(\?[^.]+)?$/.test(file); };
var isCSS = function (file) { return /\.css(\?[^.]+)?$/.test(file); };

var _a = require('chalk'), red = _a.red, yellow = _a.yellow;
var webpack = require('webpack');
var prefix = "[vue-server-renderer-webpack-plugin]";
(exports.warn = function (msg) { return console.error(red("".concat(prefix, " ").concat(msg, "\n"))); });
(exports.tip = function (msg) { return console.log(yellow("".concat(prefix, " ").concat(msg, "\n"))); });
var isWebpack5 = !!(webpack.version && webpack.version[0] > 4);
var onEmit = function (compiler, name, stageName, hook) {
    if (isWebpack5) {
        // Webpack >= 5.0.0
        compiler.hooks.compilation.tap(name, function (compilation) {
            if (compilation.compiler !== compiler) {
                // Ignore child compilers
                return;
            }
            var stage = webpack.Compilation[stageName];
            compilation.hooks.processAssets.tapAsync({ name: name, stage: stage }, function (assets, cb) {
                hook(compilation, cb);
            });
        });
    }
    else if (compiler.hooks) {
        // Webpack >= 4.0.0
        compiler.hooks.emit.tapAsync(name, hook);
    }
    else {
        // Webpack < 4.0.0
        compiler.plugin('emit', hook);
    }
};
var stripModuleIdHash = function (id) {
    if (isWebpack5) {
        // Webpack >= 5.0.0
        return id.replace(/\|\w+$/, '');
    }
    // Webpack < 5.0.0
    return id.replace(/\s\w+$/, '');
};
var getAssetName = function (asset) {
    if (typeof asset === 'string') {
        return asset;
    }
    return asset.name;
};

var hash = require('hash-sum');
var uniq = require('lodash.uniq');
var VueSSRClientPlugin = /** @class */ (function () {
    function VueSSRClientPlugin(options) {
        if (options === void 0) { options = {}; }
        //@ts-expect-error no type on options
        this.options = Object.assign({
            filename: 'vue-ssr-client-manifest.json'
        }, options);
    }
    VueSSRClientPlugin.prototype.apply = function (compiler) {
        var _this = this;
        var stage = 'PROCESS_ASSETS_STAGE_ADDITIONAL';
        onEmit(compiler, 'vue-client-plugin', stage, function (compilation, cb) {
            var stats = compilation.getStats().toJson();
            var allFiles = uniq(stats.assets.map(function (a) { return a.name; }));
            var initialFiles = uniq(Object.keys(stats.entrypoints)
                .map(function (name) { return stats.entrypoints[name].assets; })
                .reduce(function (assets, all) { return all.concat(assets); }, [])
                .map(getAssetName)
                .filter(function (file) { return isJS(file) || isCSS(file); }));
            var asyncFiles = allFiles
                .filter(function (file) { return isJS(file) || isCSS(file); })
                .filter(function (file) { return initialFiles.indexOf(file) < 0; });
            var manifest = {
                publicPath: stats.publicPath,
                all: allFiles,
                initial: initialFiles,
                async: asyncFiles,
                modules: {
                /* [identifier: string]: Array<index: number> */
                }
            };
            var assetModules = stats.modules.filter(function (m) { return m.assets.length; });
            var fileToIndex = function (asset) { return manifest.all.indexOf(getAssetName(asset)); };
            stats.modules.forEach(function (m) {
                // ignore modules duplicated in multiple chunks
                if (m.chunks.length === 1) {
                    var cid_1 = m.chunks[0];
                    var chunk = stats.chunks.find(function (c) { return c.id === cid_1; });
                    if (!chunk || !chunk.files) {
                        return;
                    }
                    var id = stripModuleIdHash(m.identifier);
                    var files_1 = (manifest.modules[hash(id)] =
                        chunk.files.map(fileToIndex));
                    // find all asset modules associated with the same chunk
                    assetModules.forEach(function (m) {
                        if (m.chunks.some(function (id) { return id === cid_1; })) {
                            files_1.push.apply(files_1, m.assets.map(fileToIndex));
                        }
                    });
                }
            });
            var json = JSON.stringify(manifest, null, 2);
            //@ts-expect-error no type on options
            compilation.assets[_this.options.filename] = {
                source: function () { return json; },
                size: function () { return json.length; }
            };
            cb();
        });
    };
    return VueSSRClientPlugin;
}());

module.exports = VueSSRClientPlugin;
