'use strict';

Object.freeze({});
/**
 * Make a map and return a function for checking if a key
 * is in that map.
 */
function makeMap(str, expectsLowerCase) {
    var map = Object.create(null);
    var list = str.split(',');
    for (var i = 0; i < list.length; i++) {
        map[list[i]] = true;
    }
    return expectsLowerCase ? function (val) { return map[val.toLowerCase()]; } : function (val) { return map[val]; };
}
/**
 * Check if a tag is a built-in tag.
 */
makeMap('slot,component', true);
/**
 * Check if an attribute is a reserved attribute.
 */
makeMap('key,ref,slot,slot-scope,is');

makeMap('accept,accept-charset,accesskey,action,align,alt,async,autocomplete,' +
    'autofocus,autoplay,autosave,bgcolor,border,buffered,challenge,charset,' +
    'checked,cite,class,code,codebase,color,cols,colspan,content,' +
    'contenteditable,contextmenu,controls,coords,data,datetime,default,' +
    'defer,dir,dirname,disabled,download,draggable,dropzone,enctype,for,' +
    'form,formaction,headers,height,hidden,high,href,hreflang,http-equiv,' +
    'icon,id,ismap,itemprop,keytype,kind,label,lang,language,list,loop,low,' +
    'manifest,max,maxlength,media,method,GET,POST,min,multiple,email,file,' +
    'muted,name,novalidate,open,optimum,pattern,ping,placeholder,poster,' +
    'preload,radiogroup,readonly,rel,required,reversed,rows,rowspan,sandbox,' +
    'scope,scoped,seamless,selected,shape,size,type,text,password,sizes,span,' +
    'spellcheck,src,srcdoc,srclang,srcset,start,step,style,summary,tabindex,' +
    'target,title,usemap,value,width,wrap');
var isJS = function (file) { return /\.js(\?[^.]+)?$/.test(file); };

var _a = require('chalk'), red = _a.red, yellow = _a.yellow;
var webpack = require('webpack');
var prefix = "[vue-server-renderer-webpack-plugin]";
var warn = (exports.warn = function (msg) { return console.error(red("".concat(prefix, " ").concat(msg, "\n"))); });
var tip = (exports.tip = function (msg) { return console.log(yellow("".concat(prefix, " ").concat(msg, "\n"))); });
var isWebpack5 = !!(webpack.version && webpack.version[0] > 4);
var validate = function (compiler) {
    if (compiler.options.target !== 'node') {
        warn('webpack config `target` should be "node".');
    }
    if (compiler.options.output) {
        if (compiler.options.output.library) {
            // Webpack >= 5.0.0
            if (compiler.options.output.library.type !== 'commonjs2') {
                warn('webpack config `output.library.type` should be "commonjs2".');
            }
        }
        else if (compiler.options.output.libraryTarget !== 'commonjs2') {
            // Webpack < 5.0.0
            warn('webpack config `output.libraryTarget` should be "commonjs2".');
        }
    }
    if (!compiler.options.externals) {
        tip('It is recommended to externalize dependencies in the server build for ' +
            'better build performance.');
    }
};
var onEmit = function (compiler, name, stageName, hook) {
    if (isWebpack5) {
        // Webpack >= 5.0.0
        compiler.hooks.compilation.tap(name, function (compilation) {
            if (compilation.compiler !== compiler) {
                // Ignore child compilers
                return;
            }
            var stage = webpack.Compilation[stageName];
            compilation.hooks.processAssets.tapAsync({ name: name, stage: stage }, function (assets, cb) {
                hook(compilation, cb);
            });
        });
    }
    else if (compiler.hooks) {
        // Webpack >= 4.0.0
        compiler.hooks.emit.tapAsync(name, hook);
    }
    else {
        // Webpack < 4.0.0
        compiler.plugin('emit', hook);
    }
};
var getAssetName = function (asset) {
    if (typeof asset === 'string') {
        return asset;
    }
    return asset.name;
};

var VueSSRServerPlugin = /** @class */ (function () {
    function VueSSRServerPlugin(options) {
        if (options === void 0) { options = {}; }
        //@ts-expect-error
        this.options = Object.assign({
            filename: 'vue-ssr-server-bundle.json'
        }, options);
    }
    VueSSRServerPlugin.prototype.apply = function (compiler) {
        var _this = this;
        validate(compiler);
        var stage = 'PROCESS_ASSETS_STAGE_OPTIMIZE_TRANSFER';
        onEmit(compiler, 'vue-server-plugin', stage, function (compilation, cb) {
            var stats = compilation.getStats().toJson();
            var entryName = Object.keys(stats.entrypoints)[0];
            var entryInfo = stats.entrypoints[entryName];
            if (!entryInfo) {
                // #5553
                return cb();
            }
            var entryAssets = entryInfo.assets.map(getAssetName).filter(isJS);
            if (entryAssets.length > 1) {
                throw new Error("Server-side bundle should have one single entry file. " +
                    "Avoid using CommonsChunkPlugin in the server config.");
            }
            var entry = entryAssets[0];
            if (!entry || typeof entry !== 'string') {
                throw new Error("Entry \"".concat(entryName, "\" not found. Did you specify the correct entry option?"));
            }
            var bundle = {
                entry: entry,
                files: {},
                maps: {}
            };
            Object.keys(compilation.assets).forEach(function (name) {
                if (isJS(name)) {
                    bundle.files[name] = compilation.assets[name].source();
                }
                else if (name.match(/\.js\.map$/)) {
                    bundle.maps[name.replace(/\.map$/, '')] = JSON.parse(compilation.assets[name].source());
                }
                // do not emit anything else for server
                delete compilation.assets[name];
            });
            var json = JSON.stringify(bundle, null, 2);
            //@ts-expect-error
            var filename = _this.options.filename;
            compilation.assets[filename] = {
                source: function () { return json; },
                size: function () { return json.length; }
            };
            cb();
        });
    };
    return VueSSRServerPlugin;
}());

module.exports = VueSSRServerPlugin;
